﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using SunVoxSharp;

namespace SunVoxPlayer
{
    public partial class FormBasic : Form
    {
        public FormBasic()
        {
            InitializeComponent();
        }

        SunVoxCore svCore;
        SunVoxTrack svTrack;
        byte[][] soundeffects;
        bool initCompleted;

        private void FormBasic_Load(object sender, EventArgs e)
        {
            initCompleted = false;
        }
        private void FormBasic_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.P)
            {
                if (initCompleted)
                {
                    MessageBox.Show("you already did that");
                    return;
                }
                label_info.Visible = false;
                Application.DoEvents();


                
                /*
                 * The code below is all you need to:
                 *   - Put library in sound-effect preload mode
                 *   - Preload a bunch of sound effects to memory
                 *   - Unload and reload library for streaming BGM
                 */
                
                svTrack = null;
                svCore = new SunVoxCore(false, false);
                soundeffects = new byte[][] {
                    svCore.render("sfx1.sunvox"),
                    svCore.render("sfx2.sunvox"),
                    svCore.render("sfx3.sunvox")
                };
                svCore.render("z.sunvox");
                svCore.dispose();
                svCore = new SunVoxCore(true, true);
                
                /* rendering z.sunvox at the end
                 * fixes a small memory leak...
                 * just pretend it's not there
                 */



                initCompleted = true;
                label_info.Visible = true;
                System.Media.SystemSounds.Asterisk.Play();
            }
            if (!initCompleted)
            {
                // haven't preloaded yet, gtfo
                return;
            }
            int playSFX = -1;
            if (e.KeyCode == Keys.Q) playSFX = 0;
            if (e.KeyCode == Keys.W) playSFX = 1;
            if (e.KeyCode == Keys.E) playSFX = 2;
            if (playSFX >= 0)
            {
                /*
                 * this uses the WinForms SoundPlayer to play the
                 * preloaded .wav files from memory, You'll probably
                 * do this very differently in XNA.
                 */
                using (MemoryStream sfx = new MemoryStream(soundeffects[playSFX]))
                {
                    new System.Media.SoundPlayer(sfx).Play();
                }
            }
            int playBGM = -1;
            if (e.KeyCode == Keys.A) playBGM = 0;
            if (e.KeyCode == Keys.S) playBGM = 1;
            if (e.KeyCode == Keys.D) playBGM = 2;
            if (playBGM >= 0)
            {
                if (svTrack != null)
                {
                    // stop and unload old song
                    svTrack.dispose();
                }
                string filename = "bgm" + (playBGM+1) + ".sunvox";
                svTrack = svCore.load(filename);
                svTrack.play();
            }
        }

        private void FormBasic_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (svCore != null)
            {
                svCore.dispose();
            }
        }
    }
}
