﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using SunVoxSharp;

namespace SunVoxPlayer
{
    public partial class FormListen : Form
    {
        public FormListen()
        {
            InitializeComponent();
        }

        double ampWidth;
        SunVoxCore svCore;
        SunVoxTrack svTrack;
        DiscoLightManager collectors;
        DiscoLightManager eventSockets;

        private void Form1_Load(object sender, EventArgs e)
        {
            // program init from another event so we can see any exceptions
            Timer t = new Timer();
            t.Tick += t_Tick;
            t.Interval = 100;
            t.Start();
        }

        void t_Tick(object sender, EventArgs e)
        {
            ((Timer)sender).Stop();
            svCore = new SunVoxCore(true, false);
            this.Text = string.Format(
                "libSunVox v{0}.{1}.{2}",
                svCore.VER_MAJOR,
                svCore.VER_MIN1,
                svCore.VER_MIN2);
            svTrack = svCore.load("gcc.sunvox");
            svTrack.play();

            // Normally you only listen to the modules you actually plan to use...
            //svTrack.listen(instrument);

            // ...but here we listen to all modules, as a stress test.
            svTrack.listen(0x01); // pluck
            svTrack.listen(0x02); // pluck vib
            svTrack.listen(0x04); // bass1
            svTrack.listen(0x05); // bass2
            svTrack.listen(0x06); // bassd
            svTrack.listen(0x07); // d.taps
            svTrack.listen(0x08); // d.bass
            svTrack.listen(0x09); // d.bass.d
            svTrack.listen(0x0a); // d.snare
            svTrack.listen(0x0b); // bass3
            svTrack.listen(0x0c); // bass4
            svTrack.listen(0x0d); // d.snare.d
            svTrack.listen(0x0e); // chord vib
            svTrack.listen(0x0f); // chord
            svTrack.listen(0x10); // echo
            svTrack.listen(0x11); // gen2
            svTrack.listen(0x12); // ghost kick
            svTrack.listen(0x13); // ghost snare

            // Prepare stuff for the kickdrum amplitude bar,
            // with its value manually read from the SunVoxTrack
            // using a relatively high resolution timer
            ampWidth = label1.Width;
            Timer tAmp = new Timer();
            tAmp.Tick += tAmp_Tick;
            tAmp.Interval = 10;
            tAmp.Start();





            // Find all labels named collect_* in the form
            // (intended targets for collectEvents),
            // and include them in the DiscoLightManager
            // for timer-based amplitude-event reads
            collectors = new DiscoLightManager(this.Controls, "collect_");

            // create a low resolution timer that
            // periodically collects all queued events
            // from sunvoxtrack and applies them to
            // the collector DiscoLightManager (GUI)
            Timer tCollect = new Timer();
            tCollect.Tick += tCollect_Tick;
            tCollect.Interval = 100;
            tCollect.Start();




            
            // event listener for directly fired events from sunvoxtrack,
            // and a DiscoLightManager to show the amplitudes in the GUI
            eventSockets = new DiscoLightManager(this.Controls, "event_");
            svTrack.ModuleSignalPeak += svTrack_ModuleSignalPeak;





            // and finally a common timer to decay the
            // colors of all disco-labels in the form
            Timer tDecay = new Timer();
            tDecay.Tick += tDecay_Tick;
            tDecay.Interval = 1;
            tDecay.Start();
        }

        void tDecay_Tick(object sender, EventArgs e)
        {
            collectors.decayColors();
            eventSockets.decayColors();
        }

        // This is called by the SunVoxTrack directly; this method
        // generates a new colour for the GUI label (based on the
        // event amplitude) and updates the GUI without timers/latency
        void svTrack_ModuleSignalPeak(object sender, SignalEvent e)
        {
            eventSockets.setLevel(e.module, e.volume);
        }

        // Instrument used for the topmost amplitude bar
        int instrument =
            //0x04; // sq.slow
            //0x0B; // saw.fast
            //0x08; // d.bass
            //0x0D; // d.snare
              0x12; //#d.bass

        // and the amplitude bar junk itself
        void tAmp_Tick(object sender, EventArgs e)
        {
            if (svTrack == null)
            {
                ((Timer)sender).Stop();
                return;
            }
            double amp = svTrack.getAmplitude(instrument);
            amp = Math.Min(amp, 1);
            int c = (int)(95+160*(1-Math.Pow(amp, 2)));
            label1.Width = (int)(ampWidth * amp);
            label2.BackColor = Color.FromArgb(c, c, c);

            /*HSL hsl = new HSL(
                0.2 * (1 - amp),
                0.5 + 0.5 * amp,
                0.4 + 0.1 * (1 - amp));
            label1.BackColor = (Color)hsl;*/
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (svCore != null)
            {
                svCore.dispose();
            }
            svTrack = null;
            svCore = null;
        }

        // collect signal events from sunvoxtrack and blink the boxes
        void tCollect_Tick(object sender, EventArgs e)
        {
            int events = 0;
            if (svTrack == null) return;
            foreach (SunVoxSharp.SignalEvent se in svTrack.collectEvents())
            {
                events++;
                collectors.setLevel(se.module, se.volume);
            }
            this.Text = "collected: " + events;
        }

        private void label12_Click(object sender, EventArgs e)
        {
            label12.Visible = false;
            label14.Visible = true;
        }

        private void label14_Click(object sender, EventArgs e)
        {
            label14.Visible = false;
            label12.Visible = true;
        }
    }
}
