﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SunVoxPlayer
{
    class Z
    {
    }
    public class HSL
    {
        public double _hue, _sat, _lum;
        public double hue
        {
            get { return _hue * scale; }
            set { _hue = CheckRange(value / scale); }
        }
        public double sat
        {
            get { return _sat * scale; }
            set { _sat = CheckRange(value / scale); }
        }
        public double lum
        {
            get { return _lum * scale; }
            set { _lum = CheckRange(value / scale); }
        }

        private const double scale = 1; //240.0;

        private double CheckRange(double value)
        {
            value = Math.Max(0, value);
            value = Math.Min(1, value);
            return value;
        }

        public override string ToString()
        {
            return String.Format("H: {0:#0.##} S: {1:#0.##} L: {2:#0.##}", hue, sat, lum);
        }

        public string ToRGBString()
        {
            Color color = (Color)this;
            return String.Format("R: {0:#0.##} G: {1:#0.##} B: {2:#0.##}", color.R, color.G, color.B);
        }

        public static implicit operator Color(HSL hsl)
        {
            double r = 0, g = 0, b = 0;
            if (hsl.lum != 0)
            {
                if (hsl.sat == 0)
                {
                    r = g = b = hsl.lum;
                }
                else
                {
                    double temp2 = GetTemp2(hsl);
                    double temp1 = 2.0 * hsl.lum - temp2;

                    r = GetColorComponent(temp1, temp2, hsl.hue + 1.0 / 3.0);
                    g = GetColorComponent(temp1, temp2, hsl.hue);
                    b = GetColorComponent(temp1, temp2, hsl.hue - 1.0 / 3.0);
                }
            }
            return Color.FromArgb((int)(255 * r), (int)(255 * g), (int)(255 * b));
        }

        private static double GetColorComponent(double temp1, double temp2, double temp3)
        {
            while (temp3 > 1.0) temp3 -= 1.0;
            while (temp3 < 0.0) temp3 += 1.0;
            if (temp3 < 1.0 / 6.0)
            {
                return temp1 + (temp2 - temp1) * 6.0 * temp3;
            }
            else if (temp3 < 0.5)
            {
                return temp2;
            }
            else if (temp3 < 2.0 / 3.0)
            {
                return temp1 + ((temp2 - temp1) * ((2.0 / 3.0) - temp3) * 6.0);
            }
            else
            {
                return temp1;
            }
        }

        private static double GetTemp2(HSL hsl)
        {
            double temp2;
            if (hsl.lum < 0.5)
            {
                temp2 = hsl.lum * (1.0 + hsl.sat);
            }
            else
            {
                temp2 = hsl.lum + hsl.sat - (hsl.lum * hsl.sat);
            }
            return temp2;
        }

        public static implicit operator HSL(Color color)
        {
            HSL hsl = new HSL();
            hsl.hue = color.GetHashCode() / 360.0;
            hsl.lum = color.GetBrightness();
            hsl.sat = color.GetSaturation();
            return hsl;
        }

        public void SetRGB(int red, int green, int blue)
        {
            HSL hsl = (HSL)Color.FromArgb(red, green, blue);
            hue = hsl.hue;
            sat = hsl.sat;
            lum = hsl.lum;
        }

        public HSL() { }
        public HSL(Color color)
        {
            SetRGB(color.R, color.G, color.B);
        }
        public HSL(int red, int green, int blue)
        {
            SetRGB(red, green, blue);
        }
        public HSL(double hue, double sat, double lum)
        {
            this.hue = hue;
            this.sat = sat;
            this.lum = lum;
        }
    }

    public class DiscoLightManager
    {
        public class Unit
        {
            public HSL color;
            public int module;
            public Label label;
        }
        public List<Unit> units;
        public DiscoLightManager(Control.ControlCollection controls, string namePrefix)
        {
            units = new List<Unit>();
            foreach (Control c in controls)
            {
                if (c.Name.StartsWith(namePrefix))
                {
                    c.ForeColor = Color.Black;
                    DiscoLightManager.Unit unit = new DiscoLightManager.Unit();
                    unit.module = Convert.ToInt32(c.Name.Substring(namePrefix.Length), 16);
                    unit.label = (Label)c;
                    unit.color = null;
                    units.Add(unit);
                }
            }
        }

        public void setLevel(int module, double amplitude)
        {
            foreach (Unit unit in units)
            {
                if (unit.module == module)
                {
                    unit.color = new HSL(
                        0.5 * (1 - amplitude),
                        0.5 + 0.5 * amplitude,
                        0.4 + 0.3 * (1 - amplitude));

                    unit.label.BackColor = (Color)unit.color;
                    break;
                }
            }
        }

        public void decayColors()
        {
            foreach (Unit unit in units)
            {
                if (unit.color == null)
                {
                    continue;
                }
                unit.color.lum = Math.Min(1.0, unit.color.lum + 0.02);
                unit.color.sat = Math.Max(0.3, unit.color.sat - 0.01);
                unit.label.BackColor = (Color)unit.color;
            }
        }
    }
}
