﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;

/* SunVoxSharp - C# binding for libSunVox
 * (c) 2012 ed <irc.rizon.net>
 * 
 * Protected by same license as libSunVox itself: source code
 * can be distributed freely, with or without changes, as long
 * as this block-comment is unmodified and present in its entirety.
 * 
 * You are free to distribute this as a binary, with or without
 * changes, as long as this text is displayed in the program's
 * about-screen or staffroll in its entirety:
 * 
 * Sound powered by SunVox modular synthesizer
 * Copyright (c) 2002 - 2012, Alexander Zolotov
 *      nightradio@gmail.com, WarmPlace.ru
 * 
 * Ogg Vorbis 'Tremor' integer playback codec
 * Copyright (c) 2002, Xiph.org Foundation
 */

namespace SunVoxSharp
{
    public class SunVoxTrack
    {
        [DllImport("sunvox.dll")] static extern int sv_open_slot(int handle);
        [DllImport("sunvox.dll")] static extern int sv_load(int handle, string filename);
        [DllImport("sunvox.dll")] static extern int sv_volume(int handle, int vol);
        [DllImport("sunvox.dll")] static extern int sv_play(int handle);
        [DllImport("sunvox.dll")] static extern int sv_play_from_beginning(int handle);
        [DllImport("sunvox.dll")] static extern int sv_rewind(int handle, int t);
        [DllImport("sunvox.dll")] static extern int sv_get_song_bpm(int handle);
        [DllImport("sunvox.dll")] static extern int sv_get_song_tpl(int handle);
        [DllImport("sunvox.dll")] static extern int sv_get_ticks();
        [DllImport("sunvox.dll")] static extern int sv_get_ticks_per_second();
        [DllImport("sunvox.dll")] static extern int sv_get_song_length_frames(int handle);
        [DllImport("sunvox.dll")] static extern int sv_get_song_length_lines(int handle);
        [DllImport("sunvox.dll")] static extern int sv_get_current_line(int handle);
        [DllImport("sunvox.dll")] static extern int sv_stop(int handle);
        [DllImport("sunvox.dll")] static extern int sv_close_slot(int handle);

        [DllImport("sunvox.dll")] static extern int sv_get_sample_type();
        [DllImport("sunvox.dll")] static extern IntPtr sv_get_module_scope(
            int slot, int mod_num, int channel, int[] offset, int[] size);

        int handle;
        int _volume;
        int _samplerate;
        int _length_rows;
        int _length_frames;
        int _ticks_per_line;
        List<bool> freeHandles;
        List<int> listenModules;
        List<double> listenLevels;
        List<SignalEvent> listenEvents;
        public event EventHandler<SignalEvent> ModuleSignalPeak;
        object listenerLock;
        bool disposed;

        public enum EventApproach
        {
            CollectEventsQueue,
            FireDirectly,
            Both
        }
        public EventApproach eventApproach;
        
        /// <summary>
        /// Create a new player and load a song
        /// (should only be used by SunVoxCore)
        /// </summary>
        /// <param name="handle">the delegated song handle (sunvox slot)</param>
        /// <param name="filepath">the file to load</param>
        public SunVoxTrack(List<bool> freeHandles, int samplerate, string filepath)
        {
            sv_open_slot(handle);
            sv_load(handle, filepath);
            this._samplerate = samplerate;
            this.freeHandles = freeHandles;
            this.handle = findFreeHandle();
            _ticks_per_line = sv_get_song_tpl(handle);
            _length_rows = sv_get_song_length_lines(handle);
            _length_frames = sv_get_song_length_frames(handle);
            using (System.IO.FileStream fsi = new System.IO.FileStream(filepath, System.IO.FileMode.Open, System.IO.FileAccess.Read))
            {
                byte[] header = new byte[0x100];
                fsi.Read(header, 0, header.Length);
                _volume = header[0x40] + header[0x41] * 256;
            }
            disposed = false;
            listenerLock = new object();
            listenModules = new List<int>();
            listenLevels = new List<double>();
            listenEvents = new List<SignalEvent>();
            eventApproach = EventApproach.Both;
            new Thread(new ThreadStart(listenLoop)).Start();
        }
        private int findFreeHandle()
        {
            int handle = freeHandles.Count;
            for (int i = 0; i < handle; i++)
            {
                if (freeHandles[i])
                {
                    freeHandles[i] = false;
                    handle = i;
                    break;
                }
            }
            if (handle == freeHandles.Count)
            {
                freeHandles.Add(false);
            }
            return handle;
        }
        void listenLoop()
        {
            int delay = 1;
            int delayRecalc = 1;
            int[] scopeSize = { 0 };
            int[] scopeOffset = { 0 };
            while (!disposed)
            {
                /*if (--delayRecalc <= 0)
                {
                    delay = (int)(1000.0 / eventTicksPerSecond);
                    delayRecalc = _ticks_per_line * 4;
                    //Console.WriteLine(engineTicksPerSecond / eventTicksPerSecond);
                    //delay = 1;
                }*/
                for (int n = 0; n < listenModules.Count; n++)
                {
                    int mod = listenModules[n];
                    double amp = 0;
                    unsafe
                    {
                        // port of soulresonance visual.pixi
                        float* data = (float*)sv_get_module_scope(handle, mod, 0, scopeOffset, scopeSize);
                        int size = 256;
                        int offs = scopeOffset[0];
                        int mask = scopeSize[0] - 1;
                        for (int a = 0; a < size; a += 8)
                        {
                            amp = Math.Max(amp, Math.Abs(data[(offs + a) & mask]));
                        }
                        // end of port
                    }
                    lock (listenerLock)
                    {
                        if (amp > listenLevels[n])
                        {
                            if (amp > listenLevels[n] * 1.2)
                            {
                                if (eventApproach != EventApproach.CollectEventsQueue)
                                {
                                    if (ModuleSignalPeak != null)
                                    {
                                        ModuleSignalPeak(this, new SignalEvent(mod, amp));
                                    }
                                }
                                if (eventApproach != EventApproach.FireDirectly)
                                {
                                    bool upgraded = false;
                                    foreach (SignalEvent se in listenEvents)
                                    {
                                        if (se.module == mod &&
                                            se.volume < amp)
                                        {
                                            se.volume = amp;
                                            upgraded = true;
                                            break;
                                        }
                                    }
                                    if (!upgraded)
                                    {
                                        listenEvents.Add(new SignalEvent(mod, amp));
                                    }
                                }
                            }
                            listenLevels[n] = amp;
                        }
                        else
                        {
                            //listenLevels[n] = Math.Max(amp, listenLevels[n] - 0.0025);
                            //listenLevels[n] = Math.Max(amp, listenLevels[n] * 0.98);
                            listenLevels[n] = Math.Max(amp, listenLevels[n] * 0.93);
                            //listenLevels[n] = amp;
                        }
                    }
                    //Console.WriteLine(amp);
                    //Console.WriteLine(scopeSize[0] + "..." + data[0]);
                }
                Thread.Sleep(delay);
            }
        }
        /// <summary>
        /// start listening to new notes on the given instrument
        /// </summary>
        public void listen(int module)
        {
            if (sv_get_sample_type() != SunVoxCore.SV_STYPE_FLOAT32)
            {
                throw new Exception("libSunVox internal renderer is not FLOAT32!");
            }
            listenLevels.Add(0);
            listenModules.Add(module);
        }
        /// <summary>
        /// get the current decaying signal peak of a module
        /// </summary>
        public double getAmplitude(int module)
        {
            lock (listenerLock)
            {
                module = listenModules.IndexOf(module);
                if (module < 0)
                {
                    throw new Exception("Cound not find requested module (remember to listen(); first)");
                }
                return listenLevels[module];
            }
        }
        /// <summary>
        /// get all events triggered by abrupt signal increases (kick/snare-drum etc)
        /// </summary>
        /// <returns>a list of all queued events</returns>
        public SignalEvent[] collectEvents()
        {
            lock (listenerLock)
            {
                if (listenModules.Count == 0)
                {
                    throw new Exception("Not listen()ing any modules!");
                }
                SignalEvent[] ret = listenEvents.ToArray();
                listenEvents.Clear();
                return ret;
            }
        }
        /// <summary>
        /// get all events triggered by abrupt signal increases (kick/snare-drum etc)
        /// </summary>
        /// <returns>a list of all queued events</returns>
        public SignalEvent[] collectEvents(int module)
        {
            lock (listenerLock)
            {
                if (listenModules.Count == 0)
                {
                    throw new Exception("Not listen()ing any modules!");
                }
                List<SignalEvent> ret = new List<SignalEvent>();
                for (int a = 0; a < listenEvents.Count; a++)
                {
                    if (listenEvents[a].module == module)
                    {
                        ret.Add(listenEvents[a]);
                        listenEvents.RemoveAt(a--);
                    }
                }
                return ret.ToArray();
            }
        }

        /// <summary>
        /// play song from the beginning
        /// </summary>
        public int play()
        {
            return sv_play_from_beginning(handle);
        }
        /// <summary>
        /// resume playback (or play from start)
        /// </summary>
        public int resume()
        {
            return sv_play(handle);
        }
        /// <summary>
        /// stop/pause playback
        /// </summary>
        public int stop()
        {
            return sv_stop(handle);
        }
        /// <summary>
        /// delete this object
        /// </summary>
        public int dispose()
        {
            if (disposed)
            {
                return 0;
            }
            disposed = true;
            freeHandles[handle] = true;
            return sv_close_slot(handle);
        }


        /// <summary>
        /// length of song in rows, constant  [read-only]
        /// </summary>
        public int length
        {
            get { return _length_rows; }
        }
        /// <summary>
        /// length of song in samples, constant  [read-only]
        /// </summary>
        public int totalSamples
        {
            get { return _length_frames; }
        }
        /// <summary>
        /// current row  [READ/WRITE]
        /// </summary>
        public int position
        {
            get { return sv_get_current_line(handle); }
            set { sv_rewind(handle, value); }
        }
        /// <summary>
        /// current volume  [READ/WRITE]
        /// </summary>
        public int volume
        {
            get { return _volume; }
            set
            {
                _volume = value;
                sv_volume(handle, _volume);
            }
        }
        /// <summary>
        /// current tick  [read-only]
        /// </summary>
        public int tick
        {
            get { return sv_get_ticks(); }
        }
        /// <summary>
        /// internal ticks per second, may change during song  [read-only]
        /// </summary>
        public int engineTicksPerSecond
        {
            get { return sv_get_ticks_per_second(); }
        }
        /// <summary>
        /// tracker ticks per second, may change during song  [read-only]
        /// </summary>
        public int eventTicksPerSecond
        {
            get { return (bpm * 4 * _ticks_per_line) / 60; }
        }
        /// <summary>
        /// Beats Per Minute  [read-only]
        /// </summary>
        public int bpm
        {
            get { return sv_get_song_bpm(handle); }
        }
    }
}
